Documentation
¶
Index ¶
Constants ¶
const Magic = uint32('A') | uint32('R')<<8 | uint32('C')<<16 | uint32(0)<<24
Magic is used in ARC-files as a "magic" number in the start of the files
const Version = "v1.0.0"
Version of the library
Variables ¶
This section is empty.
Functions ¶
This section is empty.
Types ¶
type Archive ¶
type Archive interface {
// Files should return a slice of files of the archive
Files() []File
}
Archive is type of collection of files of ARC-archives
type Chunk ¶
type Chunk interface {
// Offset should return offset of the chunk within the archive, that is
// the position of its first byte
Offset() uint32
// CompressedSize should return how many bytes the chunk occupies
// within the archive
CompressedSize() uint32
// DecompressedSize should return original size of the chunk in bytes
DecompressedSize() uint32
}
Chunk is type of file chunks within ARC-files
Directories
¶
| Path | Synopsis |
|---|---|
|
Package archive provides an implementation of arc0.Archive interface and a factory function to construct an instance of it
|
Package archive provides an implementation of arc0.Archive interface and a factory function to construct an instance of it |
|
Package chunk provides an implementation of arc0.Chunk interface and a factory function to construct an instance of it
|
Package chunk provides an implementation of arc0.Chunk interface and a factory function to construct an instance of it |
|
Package decode provides tools to extract information from files of ARC-archives
|
Package decode provides tools to extract information from files of ARC-archives |
|
archivev3
Package archivev3 provides function Pop as a mean of obtaining information on files of ARC-archives of version 3
|
Package archivev3 provides function Pop as a mean of obtaining information on files of ARC-archives of version 3 |
|
examples
|
|
|
arclist
command
|
|
|
Package file provides an implementation of arc0.File interface and a factory function to construct an instance of it
|
Package file provides an implementation of arc0.File interface and a factory function to construct an instance of it |
Click to show internal directories.
Click to hide internal directories.